if(!dojo._hasResource["com.ibm.data.DataTreeModel"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.DataTreeModel"] = true;
dojo.provide( "com.ibm.data.DataTreeModel" );

// This module provides the API for a data model that corresponds to any data set that contains
// parent-child relationships that can be represented as a tree.  It's exactly the same as the
// model API that's used by dijit.Tree with some functions already implemented for ease of use.
// As a result, any implementation of this API can be used as the model to a dijit.Tree widget
// or any other widget that uses a data model as its backend service provider.  This data API
// also adds some additional constraints and extension points that implementations should support
// in addition to the model API that's used by dijit.Tree, but by no means breaks the API or
// deviates from the required functions of the dijit model API.  One such addition is the
// support for additional query parameters that can be specified when instantiating a model
// that are always appended to any request to the server for data.  This allows particular instances
// of a data model to include metadata information that would not otherwise be included by certain
// data services that support it.

dojo.declare( "com.ibm.data.DataTreeModel",
			null,
			{
				// store: The dojo.data.Read/Write implementation 
				store: null,
				
				// query: The query used to retrieve the root item for this model
				query: null,
				
				// root: The root item of this model
				root: null,
				
				// addQueryParams: Object additional query parameters that should be appended
				//		to any request for items from the back end.  Is useful for getting
				// 		additional information such as metadata for each item from a data service 
				// 		that provides it.
				addQueryParams: null,
				
				constructor: function(args) {
					dojo.mixin(this, args);
					if(!this.addQueryParams) this.addQueryParams = {};
				},
				
				destroy: function() {
				        // summary: destroys this object, releasing connections to the store
				},
				reset: function() {
					// summary: Resets the root of this model to null.
					this.root = null;
				},
				// =======================================================================
				// Methods for traversing hierarchy
				getRoot: function(onItem, onError){
					// summary:
				    //            Calls onItem with the root item for the tree, possibly a fabricated item.
				    //			  Any request for data that is sent to the server should also include
				    //			  the request parameters in addQueryParams. 
					//            Throws exception on error.
				},
				mayHaveChildren: function(/*Object*/ item){
				        // summary:
				        //            Tells if an item has or may have children.  Implementing logic here
				        //            avoids showing +/- expando icon for nodes that we know don't have children.
				        //            (For efficiency reasons we may not want to check if an element actually
				        //            has children until user clicks the expando node)
				        
				},
				getChildren: function(/*Object*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
				        // summary:
				        // 			  Calls onComplete() with array of child items of given parent item, all loaded.
				        //			  Any request for data that is sent to the server should also include
				    	//			  the request parameters in addQueryParams. 
				        //            Throws exception on error.

				},
				// =======================================================================
				// Inspecting items
				getIdentity: function(/* item */ item){
				        // summary: returns identity for an item
				        return this.store.getIdentity(item);
				},
				getLabel: function(/*dojo.data.Item*/ item){
				        // summary: get the label for an item
				        return this.store.getLabel(item);
				},
				getParent: function(/*dojo.data.Item*/ item) {
					// summary: returns the parent item of this item argument
				},
				// =======================================================================
				// Write interface
				newItem: function(/* Object? */ args, /*Item?*/ parent){
				        // summary
				        //            Creates a new item.   See dojo.data.api.Write for details on args.
				},
				pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Item*/ nextSibling, /*Boolean?*/ extra){
				        // summary:
				        //            Move an item from one parent item to another or to another position under the same parent.
				        //            Used in drag & drop.
				        // childItem: Item to move.
				        // oldParentItem: Previous parent of item being moved.
				        // newParentItem: Item that the item being moved was dropped on.
				        // nextSiblingItem: Item that should become is the next sibling of the dropped item.  Null indicates
				        // 		dropping at the end of the list of children. 
				        // extra: Boolean flag that can be interpretted however desired by the implementing model.  Example
				        //		usages include a copy flag to indicate if the paste operation is a copy, or a flag to indicate
				        //		if the paste operation moves the child under the new item as a new child or next to the
				        //		item it was dropped on as a new sibling.

				},
				// =======================================================================
				// Callbacks
				onChange: function(/*dojo.data.Item*/ item){
				        // summary:
				        //            Callback whenever an item has changed, so that Tree
				        //            can update the label, icon, etc.   Note that changes
				        //            to an item's children or parent(s) will trigger an
				        //            onChildrenChange() so you can ignore those changes here.
				},
				onChildrenChange: function(/*dojo.data.Item*/ parent, /*dojo.data.Item[]*/ newChildrenList){
				        // summary: Callback to do notifications about new, updated, or deleted items.
				},
				onTreeChange: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Item*/ nextSiblingItem) {
					// summary: Called when the tree model is changed through one of three scenarios.
					// 		1) An item is moved in the tree model.  This can be called 
					//			when an item is reparented or reordered within the same parent.  In both
					//			cases the nextSiblingItem refers to the next sibling of childItem, sharing
					//			its same newParentItem.
					//		2) A new item is inserted into the tree at a given parent.  In this case,
					//			the oldParentItem is null.
					//		3) An item is removed from the tree at a given parent.  In this case, the
					//			newParentItem and the nextSiblingItem is null.
					// childItem: Item to move.
					// oldParentItem: Item old parent of the childItem.
					// newParentItem: Item new parent of the childItem.  If this is the same as the
					//		oldParentItem, it indicates that the childItem is being reordered in
					//		the list of children under the parent.
					// nextSiblingItem: Item that is becoming the next sibling of the moved child item.  
					// 		Null indicates moving to the end of the list of children. 
				},				
				toString: function() {
					return "[TreeModel " + this.declaredClass + "]";
				}
			}
);

}
