dojo.provide("com.ibm.pb.decorations.DecorationManager");

dojo.require("com.ibm.pb.logging.Loggable");
dojo.require("com.ibm.pb.layout.LayoutParser");
dojo.require("com.ibm.domUtilities");

dojo.declare("com.ibm.pb.decorations.DecorationManager", com.ibm.pb.logging.Loggable, {
	
	// TODO: Consolidate with com.ibm.mm.enabler.navigation.priming.PrimingCentral.COMPONENT_CONTROL_CLASS
	controlClass: "component-control",
	// TODO: Consolidate with com.ibm.mm.enabler.navigation.priming.PrimingCentral.COMPONENT_CONTAINER_CLASS
	containerClass: "component-container",
	layoutParser: com.ibm.pb.layout.LayoutParser,
	
	constructor: function() {
		this._controlComps = {};
		this._layoutComps = new Array();
		this.evtSvc = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
		this.evtSvc.subscribeEvent("com.ibm.mashups.builder.pageSwitched", this, "reset");
		this.evtSvc.subscribeEvent("com.ibm.mashups.builder.widgetDeleted", this, "unregisterControl");
		this.evtSvc.subscribeEvent("com.ibm.mashups.builder.layoutRefreshed", this, "processLayout");
	},
	processTag: function(tag){
		var mname = "processTag", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		if(dojo.hasClass(tag, this.controlClass)) {
			lgr.trace(mname, "Found control ${0}", [tag]);
			var decNode = dojo.query("a[rel=decoration],span.decoration", tag)[0];
			if(decNode) {
				lgr.trace(mname, "Found decoration ${0} on control", [decNode]);
				this.registerControl(tag, decNode);
			}
		}
		lgr.exiting(mname, arguments);
	},
    unprocessTag: function(tag) {
		if(dojo.hasClass(tag, this.controlClass)) {
			lgr.trace(mname, "Found control ${0}", [tag]);
			var id = this.layoutParser.extractID(tag);
			if(id) {
				lgr.trace(mname, "Unregister control decoration ${0}", [id]);
				this.unregisterControl(id);
			}
		}
    },
	processLayout: function(id){
		var mname = "processLayout", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		var node = this.layoutParser.findById(id);
		
		if(node && dojo.hasClass(node, this.containerClass)) {
			lgr.trace(mname, "Layout refreshed container ${0}", [node]);

			var containers = this.layoutParser.findContainers(this.layoutParser.getRootNode());
			for(var i=0; i<this._layoutComps.length; i++){
				var obj = this._layoutComps[i].obj;
				obj.processContainer(id, node, containers);				
			}
		} else {
			lgr.trace(mname, "Layout refreshed, handle all containers");
			
			if(this._layoutComps.length > 0){
				this.unregisterLayout();
			}
			dojo.query('> span.decoration', this.layoutParser.getRootNode()).forEach(dojo.hitch(this, function(node){
				this.registerLayout(node);
			}));
		}
	},
	reset: function() {
		var mname = "reset", lgr = this.LOGGER;
		lgr.entering(mname, arguments);

		var skipPageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();

		for(var controlId in this._controlComps) {
			if(this._controlComps[controlId].pageId != skipPageId)
				this.unregisterControl(controlId);
    	}

    	this.unregisterLayout(skipPageId);
    },
	registerControl: function(controlNode, decorationNode) {
    	var mname = "registerControl", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		var ret = null;

    	var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
    	var controlId = this.layoutParser.extractID(controlNode);
    	
		var name = com.ibm.domUtilities.textContent(decorationNode);
		var ctr = this.getDecorationObject(name);
		ret = this._controlComps[controlId] = {control: new ctr(controlId, controlNode), pageId: pageId};
		this.evtSvc.broadcastEvent("com.ibm.mashups.builder.skinLoaded", {widgetId: controlId, skinNodeId: controlId, skinParam: {}});

		lgr.exiting(mname, [ret]);
		return ret;
	},
	unregisterControl: function(controlId) {
		var mname = "unregisterControl", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		var ret = false;

		if(this._controlComps[controlId]) {
			if(this._controlComps[controlId].control)
				this._controlComps[controlId].control.destroy();
			delete this._controlComps[controlId];
			ret = true;
		}
		lgr.exiting(mname, [ret]);
		return ret;
	},
	registerLayout: function(node) {
		var mname = "registerLayout", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		
		var name = com.ibm.domUtilities.textContent(node);
		var cls = this.getDecorationObject(name);
		var obj = new cls();
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		this._layoutComps.push({obj: obj, pageId: pageId});
		
		var containers = this.layoutParser.findContainers(this.layoutParser.getRootNode());
		// FIXME sometimes obj.processContainer is not defined which breaks testing
		if(dojo.isFunction(obj.processContainer)) {
			for(var i=0, l = containers.length; i<l; i++) {
				var container = containers[i];
				var id = this.layoutParser.extractID(container);
				obj.processContainer(id, container);
			}
		}
		
		lgr.exiting(mname);
	},
	unregisterLayout: function(skipPageId) {
		var mname = "unregisterLayout", lgr = this.LOGGER;
		lgr.entering(mname, arguments);
		var ret = false;
		var skippedComps = new Array();
		for(var i=0; i<this._layoutComps.length; i++) {
			if (this._layoutComps[i].pageId == skipPageId) {
				skippedComps.push(this._layoutComps[i]);
			} else {
				this._layoutComps[i].obj.destroy();
			}
			ret = true;
		}
		this._layoutComps = skippedComps;
		lgr.exiting(mname, [ret]);
		return ret;
		
	},
	getDecorationObject: function(name){
		try {
			var ctr = dojo.getObject(name);
			if(!ctr) {
				dojo["require"](name);
				ctr = dojo.getObject(name);
				if(!ctr) {
					lgr.severe(mname, "${0} not found", [name]);
					return;
				}
			}
			return ctr;
		}
		catch(err) {
			lgr.severe(mname, "Error instantiating control delegate ${0}", [err]);
		}
	}
});

com.ibm.pb.decorations.DECORATION_MANAGER = new com.ibm.pb.decorations.DecorationManager();
