-- Licensed Materials - Property of IBM
-- 5724-M24
-- Copyright IBM Corporation 2011.  All Rights Reserved.
-- US Government Users Restricted Rights - Use, duplication or disclosure
-- restricted by GSA ADP Schedule Contract with IBM Corp.
-- ------------------------------------------------------------------------------------
--
-- Run the script in SQL*Plus.
--
-- USAGE:  sqlplus system/passw0rd@orcl @preMigrateBusinessSpace700.sql
--
-- ------------------------------------------------------------------------------------
-- CREATE TABLESPACE "BMTREGTABSPACE"
--    DATAFILE 'BMTREGTABSPACE' SIZE 100M REUSE
--    AUTOEXTEND ON NEXT 20M
--    EXTENT MANAGEMENT LOCAL
--    SEGMENT SPACE MANAGEMENT AUTO
--    ONLINE
--    PERMANENT
-- ;

 CREATE USER BSPMGTMP
     IDENTIFIED BY ibm_temp
--     DEFAULT TABLESPACE BMTREGTABSPACE
 ;

 GRANT CONNECT, RESOURCE to BSPMGTMP
 ;
 GRANT UNLIMITED TABLESPACE TO BSPMGTMP
 ;

-- *****************************************************************
--
-- SET SCHEMA
--
-- *****************************************************************
ALTER SESSION SET CURRENT_SCHEMA = BSPMGTMP;

-- *****************************************************************
--
-- SCHEMA VERSION (100)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.VERSION_INFO(
    OID             VARCHAR2(64)  NOT NULL,
    VERSION_MAJOR   NUMBER(10,0),
    VERSION_MINOR   NUMBER(10,0),
    VERSION_RELEASE NUMBER(10,0),
    VERSION_DEV     NUMBER(10,0),
    VERSION_TIME    NUMBER(19,0),
    DESCRIPTION     VARCHAR2(254),
    CONSTRAINT PK100 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- CATALOG (300)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG (
    OID         VARCHAR2(64)  NOT NULL,
    ACCOUNT_OID VARCHAR2(1024) NOT NULL,
    UNIQUE_NAME VARCHAR2(255) NOT NULL,
    CREATED     NUMBER(19,0)  NOT NULL,
    MODIFIED    NUMBER(19,0)  NOT NULL,
    STATE       NUMBER(10,0) DEFAULT 1 NOT NULL,
    CONSTRAINT PK300 PRIMARY KEY (OID)
);

CREATE TABLE BSPMGTMP.CATALOG_LOD (
    RESOURCE_OID VARCHAR2(64)   NOT NULL,
    LOCALE       VARCHAR2(64)   NOT NULL,
    TITLE        VARCHAR2(255)  NOT NULL,
    DESCRIPTION  VARCHAR2(1024) ,
    CONSTRAINT PK301 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK302 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.CATALOG (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- CATALOG ENTRY (310)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_ENTRY (
    OID            			VARCHAR2(64)   NOT NULL,
    CATALOG_OID    			VARCHAR2(64)   NOT NULL,
    CATEGORY_OID   			VARCHAR2(64)   NOT NULL,
    DEFINITION_URL 			VARCHAR2(2500) NOT NULL,
    CONTENT_URL    			VARCHAR2(2500) ,
    PREVIEW_URL    			VARCHAR2(2500) ,
    PREVIEW_THUMBNAIL_URL	VARCHAR2(2500)  ,
    HELP_URL       			VARCHAR2(2500)  ,    
    ICON_URL       			VARCHAR2(2500) ,
    HUB_ENTRY_URL  			VARCHAR2(2500) ,
    DEF_URL_TYPE   			NUMBER(10,0)   ,
    ICON_URL_TYPE  			NUMBER(10,0)   ,
    CREATED        			NUMBER(19,0)   NOT NULL,
    MODIFIED       			NUMBER(19,0)   NOT NULL,
    STATE          			NUMBER(10,0) DEFAULT 1 NOT NULL,
    CONSTRAINT PK310 PRIMARY KEY (OID)
);

CREATE TABLE BSPMGTMP.CATALOG_ENTRY_LOD (
    RESOURCE_OID  		VARCHAR2(64)   NOT NULL,
    LOCALE        		VARCHAR2(64)   NOT NULL,
    TITLE         		VARCHAR2(255)  NOT NULL,
    DESCRIPTION   		VARCHAR2(1024) ,
    SHORT_DESCRIPTION   VARCHAR2(255)   ,    
    CONSTRAINT PK311 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK311 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.CATALOG_ENTRY (OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.CATALOG_ENTRY_DD (
    RESOURCE_OID  VARCHAR2(64)   NOT NULL,
    NAME          VARCHAR2(255)  NOT NULL,
    VALUE         VARCHAR2(3000) ,
    CONSTRAINT PK312 PRIMARY KEY (RESOURCE_OID, NAME),
    CONSTRAINT FK312 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.CATALOG_ENTRY (OID) ON DELETE CASCADE
);


-- *****************************************************************
--
-- CATALOG CATEGORY (320)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_CATEGORY (
    OID         VARCHAR2(64)  NOT NULL,
    CATALOG_OID VARCHAR2(64)  NOT NULL,
    NAME        VARCHAR2(255) ,
    CREATED     NUMBER(19,0)  NOT NULL,
    MODIFIED    NUMBER(19,0)  NOT NULL,
    STATE       NUMBER(10,0) DEFAULT 1  NOT NULL,
    CONSTRAINT PK320 PRIMARY KEY (OID),
    CONSTRAINT FK320 FOREIGN KEY (CATALOG_OID) REFERENCES BSPMGTMP.CATALOG (OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.CATALOG_CATEGORY_LOD (
    RESOURCE_OID VARCHAR2(64)   NOT NULL,
    LOCALE       VARCHAR2(64)   NOT NULL,
    TITLE        VARCHAR2(255)  NOT NULL,
    DESCRIPTION  VARCHAR2(1024) ,
    CONSTRAINT PK321 PRIMARY KEY (RESOURCE_OID, LOCALE),
    CONSTRAINT FK321 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.CATALOG_CATEGORY (OID) ON DELETE CASCADE
);


-- *****************************************************************
--
-- CATALOG CATEGORY INCLUDES (330)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_CATEGORY_INC (
    OID          		VARCHAR2(64)    NOT NULL,
    CATALOG_OID  		VARCHAR2(64)    NOT NULL,
    CATEGORY_OID 		VARCHAR2(64)    NOT NULL,
    INCLUDE_CAT  		VARCHAR2(255)  	NOT NULL,
    INCLUDE_CATEGORY	VARCHAR2(255)   NOT NULL,
    CONSTRAINT PK330 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- CATALOG INCLUDES (340)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.CATALOG_INC (
    OID          		VARCHAR2(64)    NOT NULL,
    CATALOG_OID  		VARCHAR2(64)    NOT NULL,
    INCLUDE_CAT  		VARCHAR2(255)  	NOT NULL,
    CONSTRAINT PK340 PRIMARY KEY (OID)
);
-- *****************************************************************
--
-- PREFERENCE (400)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.PREFERENCE (
    OID          VARCHAR2(64)  NOT NULL,
    UNIQUE_NAME  VARCHAR2(255) NOT NULL,
    PARENT_OID   VARCHAR2(64)  NOT NULL,
    TYPE         NUMBER(10,0)  NOT NULL,
    WDGT_INST_ID VARCHAR2(255) NOT NULL,
    ACCOUNT_ID   VARCHAR2(1024) NOT NULL,
    PAGE_ID      VARCHAR2(255) NOT NULL,
    CREATED      NUMBER(19,0)  NOT NULL,
    MODIFIED     NUMBER(19,0)  NOT NULL,
    STATE        NUMBER(10,0) DEFAULT 1 NOT NULL,
    CONSTRAINT PK401 PRIMARY KEY (OID)
);

-- *****************************************************************
--
-- PREFERENCE NODE VALUE (410)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.PREFERENCE_VALUE (
    OID           VARCHAR2(64)      NOT NULL,
    PREF_OID      VARCHAR2(64)      NOT NULL,
    NAME          VARCHAR2(255)     NOT NULL,
    READ_ONLY     NUMBER(10,0) DEFAULT 0 NOT NULL,
    IS_LARGE      NUMBER(10,0) DEFAULT 0 NOT NULL,
    HAS_LIST      NUMBER(10,0) DEFAULT 0 NOT NULL,
    VALUE         VARCHAR2(1024)    ,
    LARGE_VALUE   CLOB              ,
    STATE         NUMBER(10,0) DEFAULT 1 NOT NULL,
    CONSTRAINT PK410 PRIMARY KEY (OID),
    CONSTRAINT FK410 FOREIGN KEY (PREF_OID) REFERENCES BSPMGTMP.PREFERENCE (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- NAVIGATION NODE (500)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.NAVNODE (
    OID            VARCHAR2(64)  NOT NULL,
    OWNER_OID      VARCHAR2(1024) NOT NULL,
    THEME_ID       VARCHAR2(255) NOT NULL,
    THEME_NAME     VARCHAR2(255) NOT NULL,
    CONTENT        BLOB          ,
    BYTE_COUNT     NUMBER(10,0)  ,
    IS_ZIPPED      NUMBER(10,0) DEFAULT 0 NOT NULL,
    COMMUNITY_OID  VARCHAR2(64)  ,
    SPACE_OID      VARCHAR2(64)  ,
    CREATED        NUMBER(19,0)  NOT NULL,
    MODIFIED       NUMBER(19,0)  NOT NULL,
    STATE          NUMBER(10,0) DEFAULT 1 NOT NULL,
    CONSTRAINT PK500 PRIMARY KEY (OID, OWNER_OID)
);

CREATE TABLE BSPMGTMP.NAVNODE_LOD (
    RESOURCE_OID  VARCHAR2(64)   NOT NULL,
    OWNER_OID     VARCHAR2(1024) NOT NULL,
    LOCALE        VARCHAR2(64)   NOT NULL,
    TITLE         VARCHAR2(255)  NOT NULL,
    DESCRIPTION   VARCHAR2(1024) ,
    CONSTRAINT FK500 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES BSPMGTMP.NAVNODE (OID,OWNER_OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.NAVNODE_DD (
    RESOURCE_OID  VARCHAR2(64)    NOT NULL,
    OWNER_OID     VARCHAR2(1024)  NOT NULL,
    NAME          VARCHAR2(255)   NOT NULL,
    VALUE         VARCHAR2(3000)  NOT NULL,
    CONSTRAINT PK502 PRIMARY KEY (RESOURCE_OID,OWNER_OID,NAME),
    CONSTRAINT FK502 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES BSPMGTMP.NAVNODE (OID,OWNER_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- NAVIGATION TREE (510)
--
-- *****************************************************************

CREATE TABLE BSPMGTMP.NAV_TREE (
    OWNER_OID    VARCHAR2(1024) NOT NULL,
    PAGE_OID     VARCHAR2(64) NOT NULL,
    PARENT_OID   VARCHAR2(64) NOT NULL,
    ORDINAL      NUMBER(10,0) DEFAULT 1 ,
    IS_VIRTUAL   NUMBER(10,0) DEFAULT 0 ,
    CONSTRAINT PK510 PRIMARY KEY (OWNER_OID, PAGE_OID)
);

-- *****************************************************************
--
-- ACCESS CONTROL (600)
--
-- *****************************************************************

CREATE TABLE BSPMGTMP.COMMUNITY_DEF (
    OID            VARCHAR2(64) NOT NULL,
    OWNER_UID      VARCHAR2(1024) NOT NULL,
    RESOURCE_OID   VARCHAR2(64) NOT NULL,
    RESOURCE_TYPE  VARCHAR2(32) NOT NULL,
    CREATED        NUMBER(19,0) NOT NULL,
    MODIFIED       NUMBER(19,0) NOT NULL,
    STATE          NUMBER(10,0) DEFAULT 1 NOT NULL,
    CONSTRAINT PK600 PRIMARY KEY( OID )
);
-- UNIQUE CONSTRAINT
ALTER TABLE BSPMGTMP.COMMUNITY_DEF ADD CONSTRAINT UQ600 UNIQUE (OID,RESOURCE_OID);

CREATE TABLE BSPMGTMP.ACL (
    COMMUNITY_OID     VARCHAR2(64)  NOT NULL,
    PARTICIPANT_UID   VARCHAR2(1024) NOT NULL,
    PARTICIPANT_TYPE  NUMBER(10,0)  NOT NULL,
    PERMISSIONS       VARCHAR2(255) NOT NULL,
    CREATED           NUMBER(19,0)  NOT NULL,
    MODIFIED          NUMBER(19,0)  NOT NULL,
    STATE             NUMBER(10,0) DEFAULT 1 NOT NULL,
    CONSTRAINT PK610 PRIMARY KEY (COMMUNITY_OID,PARTICIPANT_UID),
    CONSTRAINT FK611 FOREIGN KEY (COMMUNITY_OID) REFERENCES BSPMGTMP.COMMUNITY_DEF (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- SPACES (700)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.SPACENODE (
    OID            VARCHAR2(64)        NOT NULL,
    OWNER_OID      VARCHAR2(1024)      NOT NULL,
    ROOTNAV_OID    VARCHAR2(64)        ,
    TEMPLATE_OID   VARCHAR2(64)        ,
    CREATED        NUMBER(19,0)  NOT NULL,
    MODIFIED       NUMBER(19,0)  NOT NULL,
    STATE          NUMBER(10,0) DEFAULT 1 NOT NULL,
    CONSTRAINT PK700 PRIMARY KEY (OID, OWNER_OID)
);

CREATE TABLE BSPMGTMP.SPACENODE_LOD (
    RESOURCE_OID  VARCHAR2(64)    NOT NULL,
    OWNER_OID     VARCHAR2(1024)  NOT NULL,
    LOCALE        VARCHAR2(64)    NOT NULL,
    TITLE         VARCHAR2(255)   NOT NULL,
    DESCRIPTION   VARCHAR2(1024)          ,
    CONSTRAINT PK701 PRIMARY KEY (RESOURCE_OID,OWNER_OID,LOCALE),
    CONSTRAINT FK701 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES BSPMGTMP.SPACENODE (OID,OWNER_OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.SPACENODE_DD (
    RESOURCE_OID  VARCHAR2(64)    NOT NULL,
    OWNER_OID     VARCHAR2(1024)  NOT NULL,
    NAME          VARCHAR2(255)   NOT NULL,
    VALUE         VARCHAR2(3000)  NOT NULL,
    CONSTRAINT PK702 PRIMARY KEY (RESOURCE_OID,OWNER_OID,NAME),
    CONSTRAINT FK702 FOREIGN KEY (RESOURCE_OID,OWNER_OID) REFERENCES BSPMGTMP.SPACENODE (OID,OWNER_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- TEMPLATES (800)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.TEMPLATENODE (
    OID            VARCHAR2(64)        NOT NULL,
    OWNER_OID      VARCHAR2(1024),
    CONTENT        BLOB          ,
    BYTE_COUNT     NUMBER(10,0)  ,
    TEMPLATE_TYPE  VARCHAR2(64)  ,
    CREATED        NUMBER(19,0)  NOT NULL,
    MODIFIED       NUMBER(19,0)  NOT NULL,
    STATE          NUMBER(10,0) DEFAULT 1 NOT NULL,
    CONSTRAINT PK800 PRIMARY KEY (OID)
);

CREATE TABLE BSPMGTMP.TEMPLATENODE_LOD (
    RESOURCE_OID  VARCHAR2(64)    NOT NULL,
    LOCALE        VARCHAR2(64)    NOT NULL,
    TITLE         VARCHAR2(255)   NOT NULL,
    DESCRIPTION   VARCHAR2(1024)          ,
    CONSTRAINT PK801 PRIMARY KEY (RESOURCE_OID,LOCALE),
    CONSTRAINT FK801 FOREIGN KEY (RESOURCE_OID) REFERENCES BSPMGTMP.TEMPLATENODE (OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- ACCESS CONTROL RESOURCES (900)
--
-- *****************************************************************

CREATE TABLE BSPMGTMP.AC_RESOURCE (
    OID            VARCHAR2(64)       NOT NULL,
    OWNER_UID      VARCHAR2(1024)     NOT NULL,
    RESOURCE_OID   VARCHAR2(64)       NOT NULL,
    RESOURCE_TYPE  VARCHAR2(32)       NOT NULL,
    ROLE_BLOCKS    VARCHAR2(1024)             ,
    CREATED        NUMBER(19,0)       NOT NULL,
    MODIFIED       NUMBER(19,0)       NOT NULL,
    STATE          NUMBER(10,0)       DEFAULT 1 NOT NULL,
    CONSTRAINT PK900 PRIMARY KEY( RESOURCE_OID )
);

CREATE TABLE BSPMGTMP.AC_ROLE (
    AC_RESOURCE_OID   VARCHAR2(64)       NOT NULL,
    ROLE_OID          VARCHAR2(64)       NOT NULL,
    ROLE_NAME         VARCHAR2(1024)     NOT NULL,
    ROLE_TYPE         VARCHAR2(1024)             ,
    CREATED           NUMBER(19,0)       NOT NULL,
    MODIFIED          NUMBER(19,0)       NOT NULL,
    STATE             NUMBER(10,0)       DEFAULT 1 NOT NULL,
    CONSTRAINT PK910 PRIMARY KEY( ROLE_OID ),
    CONSTRAINT FK911 FOREIGN KEY (AC_RESOURCE_OID) REFERENCES BSPMGTMP.AC_RESOURCE (RESOURCE_OID) ON DELETE CASCADE
);

CREATE TABLE BSPMGTMP.AC_MEMBER (
    ROLE_OID          VARCHAR2(64)       NOT NULL,
    MEMBER_UID        VARCHAR2(1024)     NOT NULL,
    MEMBER_TYPE       NUMBER(10,0)       NOT NULL,
    CONSTRAINT PK920 PRIMARY KEY (ROLE_OID,MEMBER_UID),
    CONSTRAINT FK921 FOREIGN KEY (ROLE_OID) REFERENCES BSPMGTMP.AC_ROLE (ROLE_OID) ON DELETE CASCADE
);

-- *****************************************************************
--
-- FILESTORE (1000)
--
-- *****************************************************************
CREATE TABLE BSPMGTMP.FILESTORE_PATH (
    OID         VARCHAR2(64)           NOT NULL,
    PARENT_OID  VARCHAR2(64)           NOT NULL,
    NAME        VARCHAR2(255)          NOT NULL,
    PATH        VARCHAR2(255)          NOT NULL,
    LOCKED      VARCHAR2(64)           , 
    OWNER       VARCHAR2(64)           ,             
    CREATED     NUMBER(19,0)           NOT NULL,
    MODIFIED    NUMBER(19,0)           NOT NULL,
    COLL_TYPE   INTEGER DEFAULT 1      NOT NULL,
    CONSTRAINT PK1010 PRIMARY KEY (OID)
);

CREATE TABLE BSPMGTMP.FILESTORE_FILE (
    OID         VARCHAR2(64)           NOT NULL,           
    F_SIZE      NUMBER(19,0)               NOT NULL,
    CONTENT        BLOB                ,
    CONSTRAINT PK1000 PRIMARY KEY (OID),
    CONSTRAINT FK1001 FOREIGN KEY (OID) REFERENCES BSPMGTMP.FILESTORE_PATH (OID) ON DELETE CASCADE
);

--*****************************************************************
--
-- Indexing
--
--*****************************************************************
-- Catalog
CREATE INDEX BSPMGTMP.CATACCT      ON BSPMGTMP.CATALOG(ACCOUNT_OID);
CREATE INDEX BSPMGTMP.ENTRYCATGOID ON BSPMGTMP.CATALOG_ENTRY(CATALOG_OID, CATEGORY_OID);
CREATE INDEX BSPMGTMP.CATGCATOID   ON BSPMGTMP.CATALOG_CATEGORY(CATALOG_OID);
CREATE INDEX BSPMGTMP.CATINTCATOID ON BSPMGTMP.CATALOG_INC(CATALOG_OID);

-- Preferences
CREATE INDEX BSPMGTMP.PREFPARENT ON BSPMGTMP.PREFERENCE(PARENT_OID,ACCOUNT_ID);
CREATE INDEX BSPMGTMP.PREFTYPE   ON BSPMGTMP.PREFERENCE(TYPE);

-- Navigation
CREATE INDEX BSPMGTMP.PAGECOMM     ON BSPMGTMP.NAVNODE(COMMUNITY_OID);
CREATE INDEX BSPMGTMP.PAGELODOWNRS ON BSPMGTMP.NAVNODE_LOD(OWNER_OID,RESOURCE_OID);
CREATE INDEX BSPMGTMP.TREEOWPAR    ON BSPMGTMP.NAV_TREE(OWNER_OID,PARENT_OID);

-- Acl
CREATE INDEX BSPMGTMP.COMMOWNPAR ON BSPMGTMP.COMMUNITY_DEF(OWNER_UID);
CREATE INDEX BSPMGTMP.PRTTYUID   ON BSPMGTMP.ACL(PARTICIPANT_TYPE,PARTICIPANT_UID);
CREATE INDEX BSPMGTMP.PRTUID     ON BSPMGTMP.ACL(PARTICIPANT_UID);

-- AC
CREATE INDEX BSPMGTMP.AC_ROLE_RESOURCE ON BSPMGTMP.AC_ROLE(AC_RESOURCE_OID);
CREATE INDEX BSPMGTMP.AC_MEMBER_UID ON BSPMGTMP.AC_MEMBER(MEMBER_UID);
CREATE INDEX BSPMGTMP.AC_RESOURCE_OWNER ON BSPMGTMP.AC_RESOURCE(OWNER_UID,RESOURCE_OID);
